// *******************************************************************
//   Java application to build structure library for protein chains
//   
//           developed by Hui Sun Lee (huisun.cadd@gmail.com)
// ******************************************************************* 

import java.io.*;
import java.util.Vector;



public class BuildProteinStructuresLibrary{
	public static void main( String[] args ) throws IOException{
		if( args.length == 0 || args.length != 3 || args[0].equalsIgnoreCase( "--help" ) ){
			System.out.println( "Usage: BuildProteinStructuresLibrary" );
			System.out.println( "             -opts  [PDB files list]" );
			System.out.println( "                          example:  " );
			System.out.println( "                            101m.pdb" );
			System.out.println( "                            102l.pdb" );	
			System.out.println( "                            102m.pdb" );
			System.out.println( "                            ...     " );	
			System.out.println( "                    [directory where PDB files are saved]" );
			System.out.println( "                    [directory where structure library will be built]" );
			
			return;
		}
		
		
		

		String mPDBList = args[0];
		String mDirPDB = args[1];
		String mDirTarget = args[2];
		
		StructureHandler mSH = new StructureHandler();

		FileInputStream list_stream;
		BufferedReader list_reader;	
		String list_line;	
		
		DataOutputStream struct_out;

		Vector< Molecule > vMolecules_Proteins;
		
		String [] array;
		
		
        
		try{
			if( mDirPDB.charAt( mDirPDB.length() - 1 ) != '/' )
				mDirPDB = mDirPDB + "/";
			
			if( mDirTarget.charAt( mDirTarget.length() - 1 ) != '/' )
				mDirTarget = mDirTarget + "/";
				
				
				
			
			list_stream = new FileInputStream( new File( mPDBList ) );
			list_reader = new BufferedReader( new InputStreamReader( list_stream ) );

			while( ( list_line = list_reader.readLine()) != null ){
				String PDB_file = list_line;
				String PDB_id = PDB_file.substring( 0, 4 );
				
				System.out.println( PDB_file );
				
				
				// *** read molecules ***
				vMolecules_Proteins = mSH.readBiomacromoleculesFromPDBByChainId( mDirPDB + PDB_file );
				

		
				// *** remove DNA and RNA ***
				mSH.removeDNAAndRND( vMolecules_Proteins );	
				
				
				
				// *** write PDB structure files for each protein chain ***
				for( int i = 0 ; i < vMolecules_Proteins.size() ; i++ ){
					String chain_id = vMolecules_Proteins.elementAt(i).atoms.elementAt(0).chain_id;
					mSH.writeMoleculeInPDBFormat( vMolecules_Proteins.elementAt(i).atoms, mDirTarget + PDB_id + chain_id + ".pdb" );
				}	
						
											
						
				vMolecules_Proteins.removeAllElements();
			}	

			list_reader.close();
			list_stream.close();
		}
		catch( Exception exception ){
			exception.printStackTrace();
		}
	}		
}
	