// *********************************************************************************************
//   Java application to filter templates by sequence identity of their proteins to target one
//
//                     developed by Hui Sun Lee (huisun.cadd@gmail.com)
// *********************************************************************************************

import java.io.*;
import java.util.Vector;
import java.text.DecimalFormat;



public class FilterTemplatesByPositionalOverlapTanimoto{
	public static void main( String[] args ) throws IOException{
		if( args.length == 0 || args.length != 4 || args[0].equalsIgnoreCase( "--help" ) ){
			System.out.println( "Usage: FilterTemplatesByPositionalOverlapTanimoto" );
			System.out.println( "            -opts [top N templates list]" );
			System.out.println( "                  [directory where aligned template structures are saved]" );
			System.out.println( "                  [directory where non-redundant aligned template structures will be saved]" );
			System.out.println( "                  [output file]" );

			return;
		}
		
		
		
		String mListTemplates = args[0];
		String mDirTemplates = args[1];
		String mDirTarget = args[2];
		String mOutput = args[3];
		
		
		

		DecimalFormat format = new DecimalFormat( "####.###" );   //%8.3f
		format.setMinimumFractionDigits(3); 
		format.setMaximumFractionDigits(3); 



		FileInputStream list_stream;
		BufferedReader list_reader;	
		String list_line;
		
		DataOutputStream out = null;
		
		
		
		Runtime rt = Runtime.getRuntime();
		Process pr;
		LineNumberReader output_log;
		String pr_line;
		
		

		String [] array;


		

		try{
			Vector<String> vTemplateInfo = new Vector<String>();
			Vector<String> vTemplatesLig = new Vector<String>();
			
			list_stream = new FileInputStream( mListTemplates );
			list_reader = new BufferedReader( new InputStreamReader( list_stream ) );	

			while( ( list_line = list_reader.readLine()) != null ){
				array = list_line.split( "\t" );
				array = array[0].split( "-" );
				
				vTemplateInfo.add( list_line );
				vTemplatesLig.add( array[0] + "-lig-" + array[2] );				
			}

			list_reader.close();
			list_stream.close();   


				


			for( int i = 0 ; i < vTemplatesLig.size() ; i++ ){
				for( int j = i+1 ; j < vTemplatesLig.size() ; j++ ){
					double pot = getPositionalOverlapTanimoto( mDirTemplates + vTemplatesLig.elementAt(i), mDirTemplates + vTemplatesLig.elementAt(j) );

					if( pot > 0.7 ){ 
						System.out.println( vTemplatesLig.elementAt( j ) + " has been removed from templates list" );
						vTemplatesLig.remove( j );
						vTemplateInfo.remove( j );
						j--;
					}
				}
			}



			// *** write filtered list ***
			out = new DataOutputStream( new FileOutputStream( new File( mOutput ) ) );	

			for( int i = 0 ; i < vTemplateInfo.size() ; i++ ){
				out.writeBytes( vTemplateInfo.elementAt(i) + "\n" );
				
				array = vTemplateInfo.elementAt(i).split( "\t" );
				String bsres = array[0];
				array = bsres.split( "-" );
				String lig = array[0] + "-lig-" + array[2];
				
				pr = rt.exec( "cp " + mDirTemplates + bsres + " " + mDirTarget  );			
			    pr.waitFor();
				pr.getInputStream().close();
				pr.destroy(); 
				
				pr = rt.exec( "cp " + mDirTemplates + lig + " " + mDirTarget  );			
			    pr.waitFor();
				pr.getInputStream().close();
				pr.destroy(); 	
			}

			out.close();
		}
		catch( Exception exception ){
			exception.printStackTrace();
		}		
	}	






	static double getPositionalOverlapTanimoto( String lig_1, String lig_2 ){
		FileInputStream stream;
		BufferedReader reader;	
		String line;	

		String atom_name;
		double [] r = new double[3];
		double dist;

		Vector<Atom> vLig_1 = new Vector<Atom>();
		Vector<Atom> vLig_2 = new Vector<Atom>();


		try{
			// *** ligand 1 ***
			stream = new FileInputStream( new File( lig_1 ) );
			reader = new BufferedReader( new InputStreamReader( stream ) );

			while( ( line = reader.readLine()) != null ){
				if( line.startsWith( "TER" ) )
					break;

				atom_name = line.substring( 12, 16 ).trim();

				if( !atom_name.startsWith( "H" ) ){
					r[0] = Double.parseDouble( line.substring( 30, 38 ) );
					r[1] = Double.parseDouble( line.substring( 38, 46 ) );
					r[2] = Double.parseDouble( line.substring( 46, 54 ) );	

					vLig_1.add( new Atom( atom_name, r ) );
				}
			}

			reader.close();
			stream.close();

		  
			// *** ligand 2 ***
			stream = new FileInputStream( new File( lig_2 ) );
			reader = new BufferedReader( new InputStreamReader( stream ) );

			while( ( line = reader.readLine()) != null ){
				if( line.startsWith( "TER" ) )
					break;

				atom_name = line.substring( 12, 16 ).trim();

				if( !atom_name.startsWith( "H" ) ){
					r[0] = Double.parseDouble( line.substring( 30, 38 ) );
					r[1] = Double.parseDouble( line.substring( 38, 46 ) );
					r[2] = Double.parseDouble( line.substring( 46, 54 ) );	

					vLig_2.add( new Atom( atom_name, r ) );
				}
			}

			reader.close();
			stream.close();
		}
		catch( Exception exception ){
			exception.printStackTrace();
		}


        int num_lig_1 = vLig_1.size();
		int num_lig_2 = vLig_2.size();
		int num_overlap = 0;

		for( int i = 0 ; i < num_lig_1 ; i++ ){
			for( int j = 0 ; j < num_lig_2 ; j++ ){
				if( getDistance( vLig_1.elementAt(i).R, vLig_2.elementAt(j).R ) <= 1.2 ){
					num_overlap++;
					break;
				}
			}
		}

		return (double)num_overlap / ( num_lig_1 + num_lig_2 - num_overlap );
	}




	// Calculate a distance between two atoms
	public static double getDistance( double[] r1, double[] r2 ){
		return Math.sqrt( ( r1[0] - r2[0] ) * ( r1[0] - r2[0] ) + ( r1[1] - r2[1] ) * ( r1[1] - r2[1] ) + ( r1[2] - r2[2] ) * ( r1[2] - r2[2] ) );
	}
}
