// *********************************************************************************************
//   Java application to filter templates by sequence identity of their proteins to target one
//
//                     developed by Hui Sun Lee (huisun.cadd@gmail.com)
// *********************************************************************************************

import java.io.*;
import java.text.DecimalFormat;



public class FilterTemplatesBySeqIdentity{
	public static void main( String[] args ) throws IOException{
		if( args.length == 0 || args.length != 5 || args[0].equalsIgnoreCase( "--help" ) ){
			System.out.println( "FilterTemplatesBySeqIdentity" );
			System.out.println( "           -opts [target protein structure (PDB)]" );
			System.out.println( "                 [directory where PDB library protein structures are saved]" );
			System.out.println( "                 [top N templates list]" );
			System.out.println( "                 [cutoff sequence identity]" );
			System.out.println( "                 [output file]" );

			return;
		}		
		
		
		String mTargetProtein = args[0];
		String mDirLib = args[1];
		String mListTemplates = args[2];
		double mSeqId = Double.parseDouble( args[3] );
		String mOutput = args[4];
		
		
		
		
		NWAlign mNW = new NWAlign();
		
		DecimalFormat format = new DecimalFormat( "####.###" );   //%8.3f
		format.setMinimumFractionDigits(3); 
		format.setMaximumFractionDigits(3); 
		
		
		
		FileInputStream list_stream;
		BufferedReader list_reader;	
		String list_line;		

		DataOutputStream out;		

		
		String [] array;

		

		try{
			out = new DataOutputStream( new FileOutputStream( new File( mOutput ) ) );
			
			list_stream = new FileInputStream( new File( mListTemplates ) );
			list_reader = new BufferedReader( new InputStreamReader( list_stream ) );	

			while( ( list_line = list_reader.readLine()) != null ){
				array = list_line.split( "\t" );
				
				String pdb_chain = array[0].substring( 0, 5 ) + ".pdb";
				String bsres = array[0];
				
				double [] seq_identity = mNW.runNWAlign( mTargetProtein, mDirLib + pdb_chain );
				// seq_identity[0]: normalized by target protein
				// seg_identity[1]: normalized by library protein
				
				System.out.println( list_line + "\t" + "(Seq id = " + format.format( seq_identity[0] ) + ")" );

				
				if( seq_identity[0] <= mSeqId ){
					out.writeBytes( list_line + "\n" );
				}
			}

			list_reader.close();
			list_stream.close();		
			
			out.close();
		}
		catch( Exception exception ){
			exception.printStackTrace();
		}		
	}
}