// *****************************************************************
//       Java application to extract protein surface residues
//
//   This application utilizes an external program, 'VisGrid' to
//   identify surface residues.
//
//           developed by Hui Sun Lee (huisun.cadd@gmail.com)
// *****************************************************************

import java.io.*;
import java.util.Hashtable;
import java.util.Vector;



public class GetSurfaceResiduesUsingVisGrid{
	public static void main( String[] args ) throws IOException{
		if( args.length == 0 || args.length != 2 || args[0].equalsIgnoreCase( "--help" ) ){
			System.out.println( "Usage: GetSurfaceResiduesUsingVisGrid" );
			System.out.println( "            -opts  [protein structure (PDB)]" );
			System.out.println( "                   [VisGrid execution file name]" );

			return;
		}
		
		
		
		
		String mProtein = args[0];
		String mVisGrid_Exe = args[1];
		
		

		
		
		FileInputStream list_stream;
		BufferedReader list_reader;	
		String list_line;
		
		FileInputStream struct_stream;
		BufferedReader struct_reader;	
		String struct_line;
		
		DataOutputStream out;	
		

		Runtime rt = Runtime.getRuntime();
		Process pr;
		LineNumberReader output_log;
		String pr_line;
		String command;


		Hashtable<String,String> hPocketResidues = new Hashtable<String,String>();

		String [] array;		
		String value = null;



		try{
			String protein = mProtein;
			String surface = mProtein.substring( 0, mProtein.length()-4 ) + "-surface.pdb";
			
				              
            // ****************************************************************************************************************************
            // VisGrid web site: http://kiharalab.org/VisGrid/
            // Reference:
            // Bin Li, Srinivasan Turuvekere, Manish Agrawal, David La, Karthik Ramani & Daisuke Kihara
            // Characterization of local geometry of protein surfaces with the visibility criterion
            // Proteins: Struct. Funct. Bioinformatics, 71: 670-683 (2008)
            // ****************************************************************************************************************************
            
            System.out.println( "Running VisGrid for " + mProtein );
                
			pr = rt.exec( mVisGrid_Exe + " " + mProtein );			
			output_log = new LineNumberReader( new InputStreamReader( pr.getInputStream() ) );	
			
			String pocket_residues = "";

			while( ( pr_line = output_log.readLine() ) != null ){	
				if( pr_line.startsWith( "Residues" ) ){
					pocket_residues = output_log.readLine();					
					array = pocket_residues.split( "," );
					
					for( int i = 0 ; i < array.length ; i++ ){
						hPocketResidues.put( array[i].trim(), "1" );
					}
				}
			}				
			
			pr.waitFor();
			output_log.close();
			pr.getInputStream().close();
			pr.destroy(); 
               
                
                
				
				
				
				
			// *** write surface residues ***
			out = new DataOutputStream( new FileOutputStream( new File( surface ) ) );	
				
			struct_stream = new FileInputStream( new File( protein ) );
			struct_reader = new BufferedReader( new InputStreamReader( struct_stream ) );	
			
			String resSeq = "";
				
			while( ( struct_line = struct_reader.readLine()) != null ){	
				if( struct_line.startsWith( "TER" ) )
					break;
						
				resSeq = struct_line.substring( 22, 27 ).trim();
				value = hPocketResidues.get( resSeq );
					
				if( value != null )
					out.writeBytes( struct_line + "\n" );
			}
				
			out.writeBytes( "TER" + "\n" );
				
			struct_reader.close();
			struct_stream.close();              
               
            out.close();
            
            System.out.println( "Pocket residues have been written" );
		}
		catch( Exception exception ){
			exception.printStackTrace();
		}
	}
}
	