// ****************************************************************************************
//   Java application to remove structural redundancy in protein structure library
//   
//                  developed by Hui Sun Lee (huisun.cadd@gmail.com)
// **************************************************************************************** 

import java.io.*;
import java.util.Vector;
import java.util.Hashtable;



public class RemoveRedundantProteins{
	static String mList;
	static String mDirLib;
	static String mBCX;    
	static double mCutoffLigSimilarity;  
	
	
		
	public static void main( String[] args ) throws IOException{
		if( args.length == 0 || args.length != 3 || args[0].equalsIgnoreCase( "--help" ) ){
			System.out.println( "Usage: RemoveRedundantProteins" );
			System.out.println( "            -opts  [list of protein structures with resolution property]" );
			System.out.println( "                          example:  " );
			System.out.println( "                            101mA-BS-1.pdb  2.07" );
			System.out.println( "                            102lA-BS-1.pdb  1.74" );	
			System.out.println( "                            102mA-BS-1.pdb  1.84" );
			System.out.println( "                            ...     " );	
			System.out.println( "                           *X-ray resolution property can be added using AddPropertyResolution in G-LoSA Toolkit" );	
			System.out.println( "                   [bc-X.out (RCSB protein clustering result at X% sequence identity)]" );	
			System.out.println( "                       *bc-X.out files is available at http://www.rcsb.org/pdb/statistics/clusterStatistics.do" );
			System.out.println( "                   [output non-redundant file name]" );
				
			return;
		}		
		
		
		
		mList = args[0];
		mBCX = args[1];    // bc-50.out
		String mOutput = args[2];
		
		
		
		FileInputStream list_stream;
		BufferedReader list_reader;	
		String list_line;

		DataOutputStream out = null;	

		Vector<String> vProteins = new Vector<String>();
		Hashtable<String,String> hHomologousProteinChains = new Hashtable<String,String>();
		String [] array;
		

				

		try{	
			// **********************************
			//    read protein structures list
			// **********************************
			
			list_stream = new FileInputStream( new File( mList ) );
			list_reader = new BufferedReader( new InputStreamReader( list_stream ) );	
			
			while( ( list_line = list_reader.readLine()) != null ){
				vProteins.add( list_line );
			}
			
			list_reader.close();
			list_stream.close();
			
			
			
			
			
			// *******************
			//    read bc-X.out
			// *******************
			 
			list_stream = new FileInputStream( new File( mBCX ) );
			list_reader = new BufferedReader( new InputStreamReader( list_stream ) );	
			
			int bc_index = 0;
			
			while( ( list_line = list_reader.readLine()) != null ){
				bc_index++;
				
				array = list_line.split( " " );
				
				for( int i  = 0 ; i < array.length ; i++ ){
					hHomologousProteinChains.put( array[i], "" + bc_index );
				}
			}
			
			list_reader.close();
			list_stream.close();
			
			
			
			
			
			
			
			// =============
			//   Variables
			// =============
			
			int num_proteins = 0;
			
			Vector<String> vTemp = new Vector<String>();
			Vector<String> vProteins_nr = new Vector<String>();
			
			String protein_prev = "XXXXX";
			String protein_curr = "XXXXX";
			String protein_next = "XXXXX";
			
			String protein_i = null;
			String protein_j = null;
			
			double resol_i = 5.0;
			double resol_j = 5.0;
			
			String value_i = null;
			String value_j = null;
			
			
			
			
			
			// *****************************************************************************
			//    step 1: remove redundant binding site structures within individual PDBs
			// *****************************************************************************
			
			System.out.println( "\nstep 1: remove redundant protein structures within individual PDBs\n" );
			
			num_proteins = vProteins.size();
			
			int index = 0;
			
			while( true ){
				if( index == ( num_proteins - 1 ) )
					break;
					
				vTemp.removeAllElements();
				
				protein_curr = vProteins.elementAt(index);
				vTemp.add( protein_curr );
				
				System.out.println( protein_curr );
				
				while( true ){
					index++;
					protein_next = vProteins.elementAt(index);
						
					if( protein_curr.substring( 0, 4 ).equalsIgnoreCase( protein_next.substring( 0, 4 ) ) ){
						vTemp.add( protein_next );
						
						if( index == ( num_proteins - 1 ) ){
							filterVector( hHomologousProteinChains, vTemp );
							addVector( vTemp, vProteins_nr );
							break;
						}
					}
					else{
						filterVector( hHomologousProteinChains, vTemp );
						addVector( vTemp, vProteins_nr );		
						
						if( index == ( num_proteins - 1 ) )	
							vProteins_nr.add( protein_next );
			
						break;
					}	
				}
			}
			
			
			vProteins.removeAllElements();
			
			
			
			
			
			// *************************************************************************
			//    step 2: remove redundant protein structures between different PDBs
			// *************************************************************************
			
			System.out.println( "\nstep 2: remove redundant protein structures between different PDBs\n" );
			
			num_proteins = vProteins_nr.size();
			
			for( int i = 0 ; i < num_proteins ; i++ ){
				array = vProteins_nr.elementAt(i).split( "\t" );
				protein_i = array[0];
				resol_i = Double.parseDouble( array[1] );
				value_i = hHomologousProteinChains.get( protein_i.substring( 0, 4 ).toUpperCase() + "_" + protein_i.charAt(4) );
				
				System.out.println( protein_i );
				
				for( int j = i+1 ; j < num_proteins ; j++ ){
					array = vProteins_nr.elementAt(j).split( "\t" );
					protein_j = array[0];
					resol_j = Double.parseDouble( array[1] );
					value_j = hHomologousProteinChains.get( protein_j.substring( 0, 4 ).toUpperCase() + "_" + protein_j.charAt(4) );
					
					if( ( value_i != null && value_j != null ) && ( Integer.parseInt( value_i ) == Integer.parseInt( value_j ) ) ){
						if( resol_i <= resol_j ){
							vProteins_nr.removeElementAt(j);
							j--;
							num_proteins--;
						}
						else{
							vProteins_nr.removeElementAt(i);
							i--;
							num_proteins--;
							break;
						}
					}
				}
			}
			
			
			out = new DataOutputStream( new FileOutputStream( new File( mOutput ) ) );

			for( int i = 0 ; i < vProteins_nr.size() ; i++ ){
				array = vProteins_nr.elementAt(i).split( "\t" );
				out.writeBytes( array[0] + "\n" );
			}
			
			out.close();
		}
		catch( Exception exception ){
			exception.printStackTrace();
		}
	}
	
	
	
	
	
	
	
	public static void filterVector( Hashtable<String,String> hHomologousProteinChains, Vector<String> vProteins ){
		String protein_i = null;
		String protein_j = null;
			
		String value_i = null;
		String value_j = null;
		
		String [] array;
					
		for( int i = 0 ; i < vProteins.size() ; i++ ){
			array = vProteins.elementAt(i).split( "\t" );
			protein_i = array[0];
			value_i = hHomologousProteinChains.get( protein_i.substring( 0, 4 ).toUpperCase() + "_" + protein_i.charAt(4) );
				
			for( int j = i+1 ; j < vProteins.size() ; j++ ){
				array = vProteins.elementAt(j).split( "\t" );
				protein_j = array[0];
				value_j = hHomologousProteinChains.get( protein_j.substring( 0, 4 ).toUpperCase() + "_" + protein_j.charAt(4) );
					
				if( ( value_i != null && value_j != null ) && ( Integer.parseInt( value_i ) == Integer.parseInt( value_j ) ) ){
					vProteins.removeElementAt(j);
					j--;
				}
			}
		}
	}
	
	
	
	
	
	
	
	
	public static void addVector( Vector<String> vTemp, Vector<String> vProteins_nr ){
		for( int i = 0 ; i < vTemp.size() ; i++ )
			vProteins_nr.add( vTemp.elementAt(i) );
	}
}