// ******************************************************************
//      Java application to submit multiple jobs for GLoSA search
//
//           developed by Hui Sun Lee (huisun.cadd@gmail.com)
// ******************************************************************

import java.io.*;



public class RunPBSJobs{
	public static void main( String[] args ) throws IOException{
		if( args.length == 0 || args.length != 1 || args[0].equalsIgnoreCase( "--help" ) ){
			System.out.println( "Usage: RunPBSJobs" );
			System.out.println( "        -opts  [input parameter file]" );
			System.out.println( "                   *please refer to \'run_pbs_jobs.inp\' for the format of input parameter file" );


			return;
		}
		
		String mRefStruct = null;
		String mRefStructCF = null;
		String mRefSurface = null;
		String mRefSurfaceCF = null;
		String mDirLib = null;
		String mDirLibCF = null;
		String mDirList = null;
        int mIndexStartSublist = 0;
        int mIndexLastSublist = 0;
        String mDir_GLoSA_exe = null;
		double mCutoffGAScore = 1.0;
        String mDirTarget = null;
        
        String [] mParameters = new String[12];
        
        
        
        FileInputStream parameter_stream;
		BufferedReader parameter_reader;	
		String parameter_line;
        
        
		FileInputStream list_stream;
		BufferedReader list_reader;	
		String list_line;

		DataOutputStream out;	


		Runtime rt = Runtime.getRuntime();
		Process pr;
		LineNumberReader output_log;
		String pr_line;
		String command;
		
		String[] array;

        

		try{
			// *** parameter setting for G-LoSA running ***
			parameter_stream = new FileInputStream( new File( args[0] ) );
			parameter_reader = new BufferedReader( new InputStreamReader( parameter_stream ) );	
			
			int parameter_index = -1;
			
			while( ( parameter_line = parameter_reader.readLine()) != null ){
				if( parameter_line.charAt(0) == '>' ){
					parameter_index++;
					mParameters[parameter_index] = parameter_line.substring( 1, parameter_line.length() ).trim();
				}
			}
			
			parameter_reader.close();
			parameter_stream.close();
			
			mRefStruct = mParameters[0];
			mRefStructCF = mParameters[1];
			mRefSurface = mParameters[2];
			mRefSurfaceCF = mParameters[3];
			mDirLib = mParameters[4];
			mDirLibCF = mParameters[5];
			mDirList = mParameters[6];
        	mIndexStartSublist = Integer.parseInt( mParameters[7] );
        	mIndexLastSublist = Integer.parseInt( mParameters[8] );
        	mDir_GLoSA_exe = mParameters[9];
			mCutoffGAScore = Double.parseDouble( mParameters[10] );
        	mDirTarget = mParameters[11];
			
			
			
			
			
			for( int i = mIndexStartSublist ; i <= mIndexLastSublist ; i++ ){
				pr = rt.exec( "mkdir " + mDirTarget + "/search_result_" + i );				
				pr.waitFor();	
				pr.getInputStream().close();
				pr.destroy();	
				
							
				String job_name = "glosa_search_" + i;	
				String list = mDirList + "list_" + i + ".txt";
				
					
				out = new DataOutputStream( new FileOutputStream( new File( "glosa_search.inp" ) ) );
				
				out.writeBytes( "> " + mRefStruct + "\n" );
				out.writeBytes( "> " + mRefStructCF + "\n" );
				out.writeBytes( "> " + mRefSurface + "\n" );
				out.writeBytes( "> " + mRefSurfaceCF + "\n" );
				out.writeBytes( "> " + mDirLib + "\n" );
				out.writeBytes( "> " + list + "\n" );
				out.writeBytes( "> " + mDirLibCF + "\n" );
				out.writeBytes( "> " + mDir_GLoSA_exe + "\n" );
				out.writeBytes( "> " + mCutoffGAScore + "\n" );
				
				out.close();
				
				
							
				out = new DataOutputStream( new FileOutputStream( new File( job_name + ".pbs" ) ) );
					
				// ***** User Changeable PBS Script *****
				
				out.writeBytes( "#PBS -S /bin/csh" + "\n" );
				out.writeBytes( "#PBS -l nodes=1:ppn=1" + "\n" );
				out.writeBytes( "#PBS -l walltime=48:00:00" + "\n" );
				out.writeBytes( "#PBS -q default" + "\n" );
				out.writeBytes( "#PBS -N " + job_name + "\n" );
				out.writeBytes( "#PBS -o " + job_name + ".out" + "\n" );
				out.writeBytes( "#PBS -e " + job_name + ".err" + "\n" );
				out.writeBytes( "#" + "\n" );
				out.writeBytes( "\n" );
				out.writeBytes( "cd " + mDirTarget + "search_result_" + i + "\n" );					
	    		out.writeBytes( "java -XX:MaxHeapSize=64m -classpath " + mDirTarget + " GLoSASearch glosa_search.inp" + "\n" );
                
                // ***************************************
                	
				out.close();
				
				
				
				pr = rt.exec( "mv glosa_search.inp " +  mDirTarget + "/search_result_" + i + "/" );				
				pr.waitFor();	
				pr.getInputStream().close();
				pr.destroy();
				
				pr = rt.exec( "mv " + job_name + ".pbs " +  mDirTarget + "/search_result_" + i + "/" );				
				pr.waitFor();	
				pr.getInputStream().close();
				pr.destroy();	

				pr = rt.exec( "qsub " +  mDirTarget + "/search_result_" + i  + "/" + job_name + ".pbs" );				
				pr.waitFor();	
				pr.getInputStream().close();
				pr.destroy();	
			}
		}
		catch( Exception exception ){
			exception.printStackTrace();
		}
	}		
}
	